@@ -19,11 +19,13 @@ import java.io.FileOutputStream;
 import java.io.IOException;
 import java.io.PrintWriter;
 import java.io.StringWriter;
-import java.util.ArrayList;
 import java.util.Collection;
 import java.util.Collections;
 import java.util.EnumSet;
+import java.util.HashMap;
 import java.util.HashSet;
+import java.util.IdentityHashMap;
+import java.util.List;
 import java.util.Map;
 import java.util.Set;
 
@@ -34,7 +36,10 @@ import net.morilib.automata.NFA;
 import net.morilib.automata.NFAEdges;
 import net.morilib.automata.NFAState;
 import net.morilib.automata.TextBound;
+import net.morilib.automata.dfa.ConvertedRangeDFA;
 import net.morilib.automata.dfa.DFAs;
+import net.morilib.automata.nfa.RegexParseException;
+import net.morilib.automata.nfa.RegexpParser;
 import net.morilib.range.CharSets;
 import net.morilib.range.Interval;
 import net.morilib.range.Range;
@@ -52,9 +57,11 @@ public class DFABuilder implements NinaAction {
 	private static class W {
 
 		private CharSequence wrap;
+		private int quote;
 
-		W(CharSequence s) {
+		W(CharSequence s, int qc) {
 			wrap = s;
+			quote = qc;
 		}
 
 	}
@@ -69,6 +76,26 @@ public class DFABuilder implements NinaAction {
 
 	}
 
+	private static class UE {
+
+		private String edge;
+
+		UE(String s) {
+			edge = s;
+		}
+
+	}
+
+	private static class SQ {
+
+		private int num;
+
+		SQ(int n) {
+			num = n;
+		}
+
+	}
+
 	/**
 	 * 
 	 *
@@ -81,6 +108,8 @@ public class DFABuilder implements NinaAction {
 
 		TreeSectionMap<Interval, Object, DBS> edges;
 		TreeSectionMap<Interval, Object, Object> mealy;
+		Map<String, DBS> userEdge = null;
+		Map<String, Object> userMealy = null;
 		boolean initial, accepted, moore;
 		String nomFichier, label, recursiveName = null;
 		Object mealyOthers = null;
@@ -149,6 +178,15 @@ public class DFABuilder implements NinaAction {
 			}
 		}
 
+		void linkRecursive(DBS d, String name) {
+			if(others != null || end != null) {
+				throw new NinaNondeterministicException("others");
+			} else {
+				recursive = d;
+				recursiveName = name;
+			}
+		}
+
 		void linkEnd(DBS d, Object m) {
 			if(end != null || recursive != null) {
 				throw new NinaNondeterministicException("END");
@@ -158,13 +196,13 @@ public class DFABuilder implements NinaAction {
 			}
 		}
 
-		void linkRecursive(DBS d, String name) {
-			if(others != null || end != null) {
-				throw new NinaNondeterministicException("others");
-			} else {
-				recursive = d;
-				recursiveName = name;
+		void linkUser(DBS d, String l, Object m) {
+			if(userEdge == null) {
+				userEdge  = new HashMap<String, DBS>();
+				userMealy = new HashMap<String, Object>();
 			}
+			userEdge.put(l, d);
+			userMealy.put(l, m);
 		}
 
 		void sortir(Object o) {
@@ -253,6 +291,32 @@ public class DFABuilder implements NinaAction {
 			return recursiveName;
 		}
 
+		/**
+		 * 
+		 * @return
+		 */
+		public Set<String> getUserEdges() {
+			return userEdge != null ? userEdge.keySet() : null;
+		}
+
+		/**
+		 * 
+		 * @param s
+		 * @return
+		 */
+		public DBS getUserEdge(String s) {
+			return userEdge != null ? userEdge.get(s) : null;
+		}
+
+		/**
+		 * 
+		 * @param s
+		 * @return
+		 */
+		public Object getUserMealyEdge(String s) {
+			return userMealy != null ? userMealy.get(s) : null;
+		}
+
 		@Override
 		public DFAState<Object, Object, Void> go(Object a) {
 			DFAState<Object, Object, Void> d;
@@ -645,11 +709,19 @@ public class DFABuilder implements NinaAction {
 	//
 	private static final Object _END = new Object();
 
+	//
+	private Map<String, DBS> map = new HashMap<String, DBS>();
+	private Map<Object, String> inv =
+			new IdentityHashMap<Object, String>();
+	private Map<String, String> typemap =
+			new HashMap<String, String>();
 	private DBS initial, vertex;
-	private Object mealy;
 	private FA accepted = null;
+	private int mealyType;
+	private Object mealy;
+	private String name;
 
-	private DBS prendEtat(NinaEvent q) {
+	public DBS createState(NinaEvent q) {
 		DBS s;
 
 		s = new DBS();
@@ -671,7 +743,7 @@ public class DFABuilder implements NinaAction {
 		while((l = rd.read(a)) >= 0)  wr.write(a, 0, l);
 		if(vertex == null) {
 			// set initial state if it is not set
-			vertex = prendEtat(q);
+			vertex = createState(q);
 			vertex.initial = true;
 			initial = vertex;
 		}
@@ -684,98 +756,205 @@ public class DFABuilder implements NinaAction {
 		vertex.label = wr.toString();
 	}
 
+	private DBS _linkv(NinaEvent q, DBS x, DBS a, Range r, String o) {
+		x.linkAlphabet(a, r, o);
+		return a;
+	}
+
+	private DBS _linkc(NinaEvent q, DBS x, int c, String o) {
+		Range r;
+		DBS a;
+
+		r = Interval.newPoint(Integer.valueOf(c));
+		a = createState(q);
+		return _linkv(q, x, a, r, o);
+	}
+
+	private DBS _linki(NinaEvent q, DBS x, int c, String o) {
+		Range r;
+
+		r = Interval.newPoint(Integer.valueOf(c));
+		r = CharSets.complement(r);
+		return _linkv(q, x, vertex, r, o);
+	}
+
+	private void _linkdfa(NinaEvent q, DFA<Object, NFAState, Void> d,
+			DBS v, Object o) {
+		Collection<DFAState<Object, NFAState, Void>> c;
+		Map<DFAState<Object, NFAState, Void>, DBS> m;
+		DFAState<Object, NFAState, Void> h;
+		String g, lh, lc;
+		DBS x, w;
+
+		lh = q.getLanguageOption().getLookahead();
+		lc = q.getLanguageOption().getLookaheadCommit();
+		c  = DFAs.allStateRanges(d.getInitialState());
+		m  = new HashMap<DFAState<Object, NFAState, Void>, DBS>();
+		g  = lh + lc;
+		if(o != null)  g += o.toString();
+		for(DFAState<Object, NFAState, Void> y : c) {
+			if(y.equals(d.getInitialState())) {
+				x = v;
+			} else if((x = m.get(y)) == null) {
+				m.put(y, x = new DBS());
+			}
+
+			if(y.isAccepted())  x.linkOthers(vertex, g);
+			for(Interval a : y.getAlphabetRanges()) {
+				h = y.go(a.getInfimumBound());
+				if(h.equals(d.getInitialState())) {
+					x.linkAlphabet(v, a, lh);
+				} else {
+					if((w = m.get(h)) == null) {
+						m.put(h, w = new DBS());
+					}
+					x.linkAlphabet(w, a, lh);
+				}
+			}
+		}
+	}
+
+	private void _linkdq(NinaEvent q, CharSequence s, DBS v) {
+		List<DoubleQuoteTokenizer.Elems> b;
+		NFA<Object, NFAState, Void> n;
+		DFA<Object, NFAState, Void> d;
+		String g, lh, lm, lc;
+		boolean z;
+		int k, c;
+		DBS x, a;
+		Range r;
+
+		lh = q.getLanguageOption().getLookahead();
+		lm = q.getLanguageOption().getLookaheadMark();
+		lc = q.getLanguageOption().getLookaheadCommit();
+		b  = DoubleQuoteTokenizer.parse(s.toString());
+		for(DoubleQuoteTokenizer.Elems p : b) {
+			x = v;
+			s = p.getString();
+			for(k = 0; k < s.length() - 1; k++) {
+				c = s.charAt(k);
+				if(x.edges == null) {
+					x = _linkc(q, x, c, lh);
+				} else if((a = x.edges.map(c)) != null) {
+					x = a;
+				} else {
+					x = _linkc(q, x, c, lh);
+				}
+			}
+
+			if(p.getRegex() != null) {
+				x = _linkc(q, x, s.charAt(k), lh + lm);
+				try {
+					n = RegexpParser.parse(p.getRegex());
+					d = ConvertedRangeDFA.convertDFA(n);
+					_linkdfa(q, d, x, mealy);
+				} catch (RegexParseException e) {
+					throw new NinaException("invalidregex");
+				}
+			} else if(p.getLookahead() != null) {
+				x = _linkc(q, x, s.charAt(k), lh);
+				g = lm + lh;
+				z = p.isInvertLookahead();
+				s = p.getLookahead();
+				for(k = 0; k < s.length() - 1; k++) {
+					c = s.charAt(k);
+					if(z)  _linki(q, x, c, g + lc);
+					x = _linkc(q, x, c, g);
+					g = lh;
+				}
+				g = g + lc;
+				if(mealy != null)  g += mealy.toString();
+				r = Interval.newPoint(Integer.valueOf(s.charAt(k)));
+				if(p.isInvertLookahead())  r = CharSets.complement(r);
+				x.linkAlphabet(vertex, r, g);
+			} else {
+				g = lc;
+				if(mealy != null)  g += mealy.toString();
+				r = Interval.newPoint(Integer.valueOf(s.charAt(k)));
+				if(p.isInvertLookahead())  r = CharSets.complement(r);
+				x.linkAlphabet(vertex, r, g);
+			}
+		}
+	}
+
+	private void _link1(NinaEvent q, DBS v) {
+		CharSequence s;
+		int k, b, l;
+		Range r;
+		DBS x;
+
+		s = ((W)q.getEdge()).wrap;
+		b = ((W)q.getEdge()).quote;
+		l = s.length() - 1;
+		x = v;
+		if(b == '"') {
+			_linkdq(q, s, v);
+		} else {
+			for(k = 0; k < l; k++) {
+				x = _linkc(q, x, s.charAt(k), null);
+			}
+			r = Interval.newPoint(Integer.valueOf(s.charAt(k)));
+			x.linkAlphabet(vertex, r, mealy);
+		}
+	}
+
+	private void _linksq(NinaEvent q, DBS v, int len) {
+		NinaLanguageOption p = q.getLanguageOption();
+		String s, t = "", m;
+		DBS x = v, y;
+
+		m = mealy != null ? mealy.toString() : "";
+		s = mealyType == '%' ? p.getInitBuffer(m) : "";
+		for(int k = 0; k < len - 1; k++, x = y) {
+			y = createState(q);
+			if(mealyType == '%') {
+				t = p.getAppendBuffer(m);
+			} else if(mealyType == '#') {
+				t = p.getAssignArray(m, k);
+			}
+			x.linkOthers(y, s + t);
+			s = "";
+		}
+
+		if(mealyType == '%') {
+			t = p.getAppendBuffer(m);
+		} else if(mealyType == '#') {
+			t = p.getAssignArray(m, len - 1);
+		} else {
+			t = m;
+		}
+		x.linkOthers(vertex, t);
+	}
+
 	/* (non-Javadoc)
 	 * @see net.morilib.nina.NinaAction#link(net.morilib.nina.NinaEvent, java.lang.Object)
 	 */
-	@SuppressWarnings("unchecked")
 	@Override
 	public void link(NinaEvent q, Object ver) {
-		Collection<DFAState<Object, Object, Void>> c;
-		DFA<Object, Object, Void> d;
-		DBS v = vertex, x, a;
-		CharSequence s;
-		RangeAdder r;
-		Range z;
-		int k;
+		DBS v = vertex;
+		Object o;
 
-		vertex = (ver != null) ? (DBS)ver : prendEtat(q);
-		if(q.getEdge() instanceof FA) {
-			v.linkRecursive(vertex, ((FA)q.getEdge()).name);
-		} else if(q.getEdge() instanceof RT) {
-			v.linkRecursive(vertex, ((RT)q.getEdge()).name);
-		} else if(q.getEdge() instanceof DFA) {
-			d = (DFA<Object, Object, Void>)q.getEdge();
-
-			// check transition
-			c = DFAs.allStateRanges(initial);
-			for(DFAState<Object, Object, Void> y : c) {
-				if(!(y instanceof DBS)) {
-					throw new RuntimeException();
-				} else if((x = (DBS)y).edges != null) {
-					for(Tuple2<Interval, DBS> w : x.edges.entries()) {
-						if(w.getB().equals(vertex)) {
-							throw new NinaException(
-									"invaliddfasubautomaton");
-						}
-					}
-				}
-			}
-
-			// add initial state
-			x = (DBS)d.getInitialState();
-			if(x.edges != null) {
-				for(Tuple2<Interval, DBS> w : x.edges.entries()) {
-					v._linkRange(w.getB(), w.getA(), w.getA(),
-							x.mealy.map(w.getA()));
-				}
-			}
-
-			// change accept states
-			c = DFAs.allAcceptedStateRanges(d);
-			r = new RangeAdder();
-			vertex.identical = new ArrayList<DBS>();
-			if(vertex.edges != null) {
-				for(Tuple2<Interval, DBS> w : vertex.edges.entries()) {
-					r.add(w.getA());
-				}
-			}
-			z = r.toRange();
-
-			for(DFAState<Object, Object, Void> y : c) {
-				if(!(y instanceof DBS)) {
-					throw new RuntimeException();
-				} else if(((DBS)y).identical != null) {
-					throw new NinaException("invaliddfasubautomaton");
-				}
-
-				if((x = (DBS)y).edges != null) {
-					for(Tuple2<Interval, DBS> w : x.edges.entries()) {
-						x._linkRange(w.getB(), z, w.getA(),
-								x.mealy.map(w.getA()));
-					}
-				}
-				x.accepted = false;
-				vertex.identical.add(x);
-			}
-		} else if(q.getEdge() instanceof W) {
-			s = ((W)q.getEdge()).wrap;
-			for(x = v, k = 0; k < s.length() - 1; k++, x = a) {
-				a = prendEtat(q);
-				x.linkAlphabet(a,
-						Interval.newPoint(Integer.valueOf(s.charAt(k))),
-						null);
-			}
-			x.linkAlphabet(vertex,
-					Interval.newPoint(Integer.valueOf(s.charAt(k))),
-					null);
-		} else if(q.getEdge().equals(CharSets.ALL_CHAR)) {
+		vertex = (ver != null) ? (DBS)ver : createState(q);
+		if((o = q.getEdge()) == null) {
+			v.linkOthers(vertex, q.getLanguageOption().getUnget());
+		} else if(o instanceof FA) {
+			v.linkRecursive(vertex, ((FA)o).name);
+		} else if(o instanceof RT) {
+			v.linkRecursive(vertex, ((RT)o).name);
+		} else if(o instanceof W) {
+			_link1(q, v);
+		} else if(o instanceof SQ) {
+			_linksq(q, v, ((SQ)o).num);
+		} else if(o.equals(CharSets.ALL_CHAR)) {
 			v.linkOthers(vertex, mealy);
-		} else if(q.getEdge() instanceof Range) {
-			v.linkAlphabet(vertex, (Range)q.getEdge(), mealy);
-		} else if(q.getEdge() == _END) {
+		} else if(o instanceof Range) {
+			v.linkAlphabet(vertex, (Range)o, mealy);
+		} else if(o instanceof UE) {
+			v.linkUser(vertex, ((UE)o).edge, mealy);
+		} else if(o == _END) {
 			v.linkEnd(vertex, mealy);
 		} else {
-			v.linkAlphabet(vertex, q.getEdge(), mealy);
+			v.linkAlphabet(vertex, o, mealy);
 		}
 		mealy = null;
 	}
@@ -809,7 +988,11 @@ public class DFABuilder implements NinaAction {
 	 */
 	@Override
 	public void doneBlockSearching(NinaEvent q) {
-		vertex = (DBS)q.getScratch();
+		if(q.getScratch() instanceof NinaParser.Z) {
+			vertex = (DBS)((NinaParser.Z)q.getScratch()).vertex;
+		} else {
+			vertex = (DBS)q.getScratch();
+		}
 	}
 
 	/* (non-Javadoc)
@@ -824,8 +1007,9 @@ public class DFABuilder implements NinaAction {
 	 * @see net.morilib.nina.NinaAction#setEdgeCharSequence(net.morilib.nina.NinaEvent, java.lang.CharSequence)
 	 */
 	@Override
-	public void setEdgeCharSequence(NinaEvent q, CharSequence o) {
-		q.setEdge(new W(o));
+	public void setEdgeCharSequence(NinaEvent q, CharSequence o,
+			int qc) {
+		q.setEdge(new W(o, qc));
 	}
 
 	/* (non-Javadoc)
@@ -843,7 +1027,8 @@ public class DFABuilder implements NinaAction {
 			q.setEdge(new RT(t));
 		} else {
 			sub.put(t, null);
-			q.setEdge(p._compileRes(s, new DFABuilder(), map, sub));
+			q.setEdge(p.compileSubautomaton(s, new DFABuilder(), map,
+					sub).getMachine());
 		}
 	}
 
@@ -868,10 +1053,15 @@ public class DFABuilder implements NinaAction {
 	 * @see net.morilib.nina.NinaAction#accept()
 	 */
 	@Override
-	public Object accept(String name) {
+	public NinaAction accept(String name) {
 		if(accepted == null) {
+			this.name = name;
 			accepted = new FA(name, initial);
 		}
+		return this;
+	}
+
+	public DFA<Object, Object, Void> getMachine() {
 		return accepted;
 	}
 
@@ -899,4 +1089,103 @@ public class DFABuilder implements NinaAction {
 		// do nothing
 	}
 
+	/* (non-Javadoc)
+	 * @see net.morilib.nina.NinaAction#putStateByLabel(java.lang.String, java.lang.Object)
+	 */
+	@Override
+	public void putStateByLabel(String l, Object s) {
+		map.put(l, (DBS)s);
+		inv.put(s, l);
+	}
+
+	/* (non-Javadoc)
+	 * @see net.morilib.nina.NinaAction#getStateByLabel(java.lang.String)
+	 */
+	@Override
+	public DBS getStateByLabel(String l) {
+		return map.get(l);
+	}
+
+	/* (non-Javadoc)
+	 * @see net.morilib.nina.NinaAction#getLabelByState(java.lang.Object)
+	 */
+	@Override
+	public String getLabelByState(Object s) {
+		return inv.get(s);
+	}
+
+	/* (non-Javadoc)
+	 * @see net.morilib.nina.NinaAction#getDeadState()
+	 */
+	@Override
+	public Object getDeadState() {
+		return getStateByLabel(NinaParser.DEADSYM);
+	}
+
+	/* (non-Javadoc)
+	 * @see net.morilib.nina.NinaAction#setStartState(java.lang.Object)
+	 */
+	@Override
+	public void setStartState(Object o) {
+		vertex = (DBS)o;
+		vertex.initial = true;
+		initial = vertex;
+	}
+
+	/* (non-Javadoc)
+	 * @see net.morilib.nina.NinaAction#setUserEdge(net.morilib.nina.NinaEvent, java.lang.String)
+	 */
+	@Override
+	public void setUserEdge(NinaEvent q, String o) {
+		q.setEdge(new UE(o));
+	}
+
+	/* (non-Javadoc)
+	 * @see net.morilib.nina.NinaAction#getLabels()
+	 */
+	@Override
+	public Set<String> getLabels() {
+		return map.keySet();
+	}
+
+	/* (non-Javadoc)
+	 * @see net.morilib.nina.NinaAction#putTypeByLabel(java.lang.String, java.lang.Object)
+	 */
+	@Override
+	public void putTypeByLabel(String l, String s) {
+		typemap.put(l, s);
+	}
+
+	/* (non-Javadoc)
+	 * @see net.morilib.nina.NinaAction#getTypeByLabel(java.lang.String)
+	 */
+	@Override
+	public String getTypeByLabel(String l) {
+		return typemap.get(l);
+	}
+
+	/* (non-Javadoc)
+	 * @see net.morilib.nina.NinaAction#getName()
+	 */
+	@Override
+	public String getName() {
+		return name;
+	}
+
+	/* (non-Javadoc)
+	 * @see net.morilib.nina.NinaAction#setEdgeSequence(net.morilib.nina.NinaEvent)
+	 */
+	@Override
+	public void setEdgeSequence(NinaEvent q, int len) {
+		q.setEdge(new SQ(len));
+	}
+
+	/* (non-Javadoc)
+	 * @see net.morilib.nina.NinaAction#setMealyEdgeType(int)
+	 */
+	@Override
+	public void setMealyEdgeType(int c) {
+		mealyType = c;
+	}
+
 }
